library(mgcv)
library(survival)
library(ggplot2)
library(gridExtra)
library(reshape2)
library(pryr)
library(devtools)

setwd(".") # Set to directory with package source
install.packages("pcox_0.1.0.tar.gz", repos=NULL, type="source")
library(pcox)

data(sofa_fu)
N <- nrow(sofa_fu)
J <- ncol(sofa_fu$SOFA)
class(sofa_fu[[3]]) <- NULL
class(sofa_fu[[4]]) <- NULL
class(sofa_fu[[5]]) <- NULL
sofa_fu$log.los <- log(sofa_fu$los)

xind <- seq(log(2), log(157), length=101)
tind <- seq(0,1,length=101)
fhat.dat <- data.frame(log.los=xind)
bhat.dat <- data.frame(smat=tind, LX=1)
  
fits.sofa <- list()
fhat = bhat <- matrix(nrow=3, ncol=101)
Zhat <- matrix(nrow=3, ncol=3)
methods <- c("aic", "caic", "epic")
for (m in 1:3) {
  fit <- pcox(Surv(time,event) ~ age + male + Charlson +
                p(log.los, linear=FALSE, bs="ps", k=15) +
                bf(SOFA_ds, bs="ps", m=c(2,2), k=20),
              data=sofa_fu, method=methods[m], eps=.000001,
              iter.max=100, outer.max=100)
  fits.sofa[[m]] <- fit
  fhat[m,] <- PredictMat(fit$pcox$smooth[[1]][[1]], data=fhat.dat) %*% fit$coef[4:17]
  bhat[m,] <- PredictMat(fit$pcox$smooth[[2]][[1]], data=bhat.dat) %*% fit$coef[18:37]
  Zhat[m,] <- fit$coef[1:3]
}

fhat.sd1 <- t(sapply(fits.sofa, function(fit) {
  xp <- PredictMat(fit$pcox$smooth[[1]][[1]], data = fhat.dat)
  sqrt(diag(xp %*% fit$var[4:17,4:17] %*% t(xp)))
}))
fhat.sd2 <- t(sapply(fits.sofa, function(fit) {
  xp <- PredictMat(fit$pcox$smooth[[1]][[1]], data = fhat.dat)
  sqrt(diag(xp %*% fit$var2[4:17,4:17] %*% t(xp)))
}))
bhat.sd1 <- t(sapply(fits.sofa, function(fit) {
  xp <- PredictMat(fit$pcox$smooth[[2]][[1]], data = bhat.dat)
  sqrt(diag(xp %*% fit$var[18:37,18:37] %*% t(xp)))
}))
bhat.sd2 <- t(sapply(fits.sofa, function(fit) {
  xp <- PredictMat(fit$pcox$smooth[[2]][[1]], data = bhat.dat)
  sqrt(diag(xp %*% fit$var2[18:37,18:37] %*% t(xp)))
}))


# Bootstrap CI's

B <- 10000
samp.boot <- matrix(nrow=B, ncol=N)
fhat.boot <- array(dim=c(B,3,length(xind)))
bhat.boot <- array(dim=c(B,3,length(tind)))
Zhat.boot <- array(dim=c(B,3,3))
rownames(sofa_fu) <- NULL
for (i in 1:length(sofa_fu)) if (!is.null(dim(sofa_fu[[i]])))
  rownames(sofa_fu[[i]]) <- NULL

for (b in 1:B) {
  if ((b/10) == round(b/10))
    print(b)
  
  samp <- sample(1:N, replace=TRUE)
  samp.boot[b,] <- samp
  dat.b <- sofa_fu[samp,]
  for (m in 1:3) {
    fit.b <- pcox(Surv(time,event) ~ age + male + Charlson +
                  p(log.los, linear=FALSE, bs="ps", k=15) +
                  bf(SOFA_ds, bs="ps", m=c(2,2), k=20),
                data=dat.b, method=methods[m], eps=.000001,
                iter.max=100, outer.max=100)
    
    fhat.boot[b,m,] <- PredictMat(fit.b$pcox$smooth[[1]][[1]], data=fhat.dat
    ) %*% fit.b$coef[4:17]
    bhat.boot[b,m,] <- PredictMat(fit.b$pcox$smooth[[2]][[1]], data=bhat.dat
    ) %*% fit.b$coef[18:37]
    Zhat.boot[b,m,] <- fit.b$coef[1:3]
  }
}

# save(fhat, fhat.boot, bhat, bhat.boot, Zhat, Zhat.boot,
#      file="bootResults.RData")
# load("bootResults.RData")

fhat.sd3 <- apply(fhat.boot, 2:3, sd, na.rm=TRUE)
bhat.sd3 <- apply(bhat.boot, 2:3, sd, na.rm=TRUE)

fhat.long <- melt(fhat, varnames=c("method", "log.los"))[,c(2,1,3)]
fhat.long$log.los <- xind[fhat.long$log.los]
bhat.long <- melt(bhat, varnames=c("method", "s"))[,c(2,1,3)]
bhat.long$s = tind[bhat.long$s]
fhat.long$method = bhat.long$method <- factor(fhat.long$method,
                                              labels=c("AIC", "AIC[c]", "EPIC"))

fhat.ci.long <- do.call("rbind", lapply(1:3, function(i) {
  fhat.sd <- get(paste0("fhat.sd",i))
  fhat.ci <- melt(sapply(1:3, function(m) {
    cbind(fhat[m,] - 1.96*fhat.sd[m,], fhat[m,] + 1.96*fhat.sd[m,])
  }, simplify="array"), varnames=c("log.los", "uplow", "method"))
  fhat.ci$CIType <- c("V1", "V2", "B-V")[i]
  fhat.ci
}))

bhat.ci.long <- do.call("rbind", lapply(1:3, function(i) {
  bhat.sd <- get(paste0("bhat.sd",i))
  bhat.ci <- melt(sapply(1:3, function(m) {
    cbind(bhat[m,] - 1.96*bhat.sd[m,], bhat[m,] + 1.96*bhat.sd[m,])
  }, simplify="array"), varnames=c("s", "uplow", "method"))
  bhat.ci$CIType <- c("V1", "V2", "B-V")[i]
  bhat.ci
}))

fhat.ci4 <- apply(fhat.boot, 2:3, quantile, probs=c(.025,.975), na.rm=TRUE)
bhat.ci4 <- apply(bhat.boot, 2:3, quantile, probs=c(.025,.975), na.rm=TRUE)
dimnames(fhat.ci4) = dimnames(bhat.ci4) <- NULL
fhat.ci4.long <- melt(fhat.ci4, varnames=c("uplow", "method", "log.los"))[,c(3,1,2,4)]
bhat.ci4.long <- melt(bhat.ci4, varnames=c("uplow", "method", "s"))[,c(3,1,2,4)]
fhat.ci4.long$CIType = bhat.ci4.long$CIType <- "B-Q"
fhat.ci.long <- rbind(fhat.ci.long, fhat.ci4.long)
bhat.ci.long <- rbind(bhat.ci.long, bhat.ci4.long)

fhat.ci.long$log.los <- xind[fhat.ci.long$log.los]
fhat.ci.long$method <- factor(fhat.ci.long$method, labels=c("AIC", "AIC[c]", "EPIC"))
fhat.ci.long$CIType <- factor(fhat.ci.long$CIType, levels=c("V1","V2","B-V","B-Q"))
bhat.ci.long$s      <- tind[bhat.ci.long$s]
bhat.ci.long$method <- factor(bhat.ci.long$method, labels=c("AIC", "AIC[c]", "EPIC"))
bhat.ci.long$CIType <- factor(bhat.ci.long$CIType, levels=c("V1","V2","B-V","B-Q"))

fhat.long$los <- exp(fhat.long$log.los)
fhat.ci.long$los <- exp(fhat.ci.long$log.los)

plot(fhat.long$los, fhat.long$valuevalue, log="x")
breaks <- axTicks(side=1)

fhat.plot <- ggplot(fhat.long, aes(los,value)) +
  scale_x_continuous(trans="log", breaks = breaks) +
  coord_cartesian(ylim=c(-5,5), xlim=c(2,157)) +
  ylab(expression(paste(hat(f)(log~~U[i])))) +
  #   xlab(expression(paste(log~~U[i]))) +
  xlab(expression(paste(U[i]))) +
  geom_hline(aes(yintercept=0), colour="green") +
  geom_line(size=2) +
  facet_grid(.~method, labeller = label_parsed) +
  geom_line(aes(group=interaction(CIType,uplow), colour=CIType), 
            data=fhat.ci.long) +
  labs(colour="CI Type")

bhat.plot <- ggplot(bhat.long, aes(s,value)) +
  coord_cartesian(ylim=c(-10,10)) +
  ylab(expression(paste(hat(beta)(s)))) +
  geom_hline(aes(yintercept=0), colour="green") +
  geom_line(size=2) +
  facet_grid(.~method, labeller = label_parsed) +
  geom_line(aes(group=interaction(CIType,uplow), colour=CIType), 
            data=bhat.ci.long) +
  labs(colour="CI Type")

g_legend<-function(p){
  tmp <- ggplotGrob(p)
  leg <- which(sapply(tmp$grobs, function(x) x$name) == "guide-box")
  legend <- tmp$grobs[[leg]]
  return(legend)}
est.leg <- g_legend(bhat.plot)
lwidth <- sum(est.leg$width)

## using grid.arrange for convenience
## could also manually push viewports
estGrob <- arrangeGrob(fhat.plot + theme(legend.position="none"),
                       bhat.plot + theme(legend.position="none"),
                       nrow=2)
estFig <- arrangeGrob(estGrob, arrangeGrob(est.leg), nrow=1,
                      widths=unit.c(unit(1, "npc") - lwidth, lwidth)
)
ggsave("gg_fbhat.eps", plot=estFig, width=11, height=6.5)


# Kaplan-Meier Plots

createSurvivalFrame <- function(f.survfit) {
  # initialise frame variable
  f.frame <- NULL
  if(length(names(f.survfit$strata)) == 0) {
    # create data.frame with data from survfit
    f.frame <- data.frame(time=f.survfit$time, n.risk=f.survfit$n.risk, n.event=f.survfit$n.event, n.censor = f.survfit$n.censor, surv=f.survfit$surv, upper=f.survfit$upper, lower=f.survfit$lower)
    f.start <- data.frame(time=c(0, f.frame$time[1]), n.risk=c(f.survfit$n, f.survfit$n), n.event=c(0,0), n.censor=c(0,0), surv=c(1,1), upper=c(1,1), lower=c(1,1))
    f.frame <- rbind(f.start, f.frame)
    rm(f.start)
  } else {
    # create vector for strata identification
    f.strata <- NULL
    for(f.i in 1:length(f.survfit$strata)){
      f.strata <- c(f.strata, rep(names(f.survfit$strata)[f.i], f.survfit$strata[f.i]))
    }
    f.frame <- data.frame(time=f.survfit$time, n.risk=f.survfit$n.risk, n.event=f.survfit$n.event, n.censor = f.survfit$n.censor, surv=f.survfit$surv, upper=f.survfit$upper, lower=f.survfit$lower, strata=factor(f.strata))
    rm(f.strata)
    for(f.i in 1:length(f.survfit$strata)){
      f.subset <- subset(f.frame, strata==names(f.survfit$strata)[f.i])
      f.start <- data.frame(time=c(0, f.subset$time[1]), n.risk=rep(f.survfit[f.i]$n, 2), n.event=c(0,0), n.censor=c(0,0), surv=c(1,1), upper=c(1,1), lower=c(1,1), strata=rep(names(f.survfit$strata)[f.i], 2))
      f.frame <- rbind(f.start, f.frame)
      rm(f.start, f.subset)
    }
    f.frame <- f.frame[order(f.frame$strata, f.frame$time), ]
    rownames(f.frame) <- NULL
  }
  return(f.frame)
}

# define custom function to draw kaplan-meier curve with ggplot
qplot_survival <- function(f.frame, f.CI="default", f.shape=3) {
  # use different plotting commands dependig whether or not strata's are given
  if("strata" %in% names(f.frame) == FALSE){
    # confidence intervals are drawn if not specified otherwise
    if(f.CI=="default" | f.CI==TRUE ){
      # create plot with 4 layers (first 3 layers only events, last layer only censored)
      # hint: censoring data for multiple censoring events at timepoint are overplotted
      # (unlike in plot.survfit in survival package)
      
      ggplot(data=f.frame) +
        geom_step(aes(x=time, y=surv), direction="hv") +
        geom_step(aes(x=time, y=upper), directions="hv", linetype=2) +
        geom_step(aes(x=time,y=lower), direction="hv", linetype=2) +
        geom_point(data=subset(f.frame, n.censor==1), aes(x=time, y=surv),
                   shape=f.shape)
    } else {
      # create plot without confidence intervalls
      ggplot(data=f.frame) +
        geom_step(aes(x=time, y=surv), direction="hv") +
        geom_point(data=subset(f.frame, n.censor==1), aes(x=time, y=surv),
                   shape=f.shape)
    }
  } else {
    if(f.CI=="default" | f.CI==FALSE){
      # without CI
      ggplot(data=f.frame, aes(group=strata, colour=strata)) +
        geom_step(aes(x=time, y=surv), direction="hv") +
        geom_point(data=subset(f.frame, n.censor==1), aes(x=time, y=surv),
                   shape=f.shape)
    } else {
      # with CI (hint: use alpha for CI)
      ggplot(data=f.frame, aes(colour=strata, group=strata)) +
        geom_step(aes(x=time, y=surv), direction="hv") +
        geom_step(aes(x=time, y=upper), directions="hv", linetype=2, alpha=0.5) +
        geom_step(aes(x=time,y=lower), direction="hv", linetype=2, alpha=0.5) +
        geom_point(data=subset(f.frame, n.censor==1), aes(x=time, y=surv),
                   shape=f.shape)
    }
  }
}


# K-M Plots
X <- sofa_fu$SOFA_ds
tind2<- seq(0,1,length=J)
sofa_fu$M1 = rowMeans(X[, 1:round(0.05*J)])
sofa_fu$M2 = rowMeans(X[, round(0.05*J):round(0.2*J)])
sofa_fu$M3 = rowMeans(X[, round(0.20*J):round(0.4*J)])
sofa_fu$M4 = rowMeans(X[, round(0.85*J):round(1.0*J)])

sofa_fu$highM1 <- sofa_fu$M1>median(sofa_fu$M1)
sofa_fu$highM4 <- sofa_fu$M4>median(sofa_fu$M4)

sf   <- survfit(Surv(time,event) ~ highM4 + highM1, data=sofa_fu)
fit.km <- coxph(Surv(time,event) ~ highM4*highM1, data=sofa_fu)
sf1  <- survfit(Surv(time,event) ~ highM1, data=sofa_fu)
sf2  <- survfit(Surv(time,event) ~ highM4, data=sofa_fu)

sf1.frame <- createSurvivalFrame(sf1)
sf2.frame <- createSurvivalFrame(sf2)
sf.frame  <- createSurvivalFrame(sf)

sf1.frame$plot <- "M1"
sf2.frame$plot <- "M4"
sf.frame$plot  <- "M1 + M4"

all.frame <- rbind(sf1.frame, sf2.frame, sf.frame)
all.frame$plot <- factor(all.frame$plot, levels=c("M1", "M4", "M1 + M4"))

m1.plot <- qplot_survival(sf1.frame) +
  ylim(0,1) +
  labs(y=expression(paste(hat(S)(t))), x="t") +
  theme(legend.justification=c(0,0), legend.position=c(0,0)) +
  scale_colour_discrete(name="p=0.115",
                        breaks=c("highM1=FALSE", "highM1=TRUE"),
                        labels=c("Low M1", "High M1"))

m4.plot <- qplot_survival(sf2.frame) +
  ylim(0,1) +
  labs(y=expression(paste(hat(S)(t))), x="t") +
  theme(legend.justification=c(0,0), legend.position=c(0,0)) +
  scale_colour_discrete(name="p=0.006",
                        breaks=c("highM4=FALSE", "highM4=TRUE"),
                        labels=c("Low M4", "High M4"))

m14.plot <- qplot_survival(sf.frame) +
  ylim(0,1) +
  labs(y=expression(paste(hat(S)(t))), x="t") +
  theme(legend.justification=c(0,0), legend.position=c(0,0)) +
  scale_colour_discrete(name="p=0.012",
                        breaks=levels(sf.frame$strata)[4:1],
                        labels=c("Low M1, Low M4", "High M1, Low M4",
                                 "Low M1, High M4", "High M1, High M4"))

all.plot <- qplot_survival(all.frame) + facet_grid(.~plot)

mfig <- arrangeGrob(m1.plot, m4.plot, m14.plot, nrow=1)
ggsave("gg_surv.eps", plot=mfig, width=11, height=4)
